; File name		:	tdyspkr.asm
; Project name	:	Tandy 1000 SX Speaker Multiplexer (tdyspkr.com)
; Created date	:	4.10.2009
; Last update	:	4.10.2009
; Author		:	Tomi Tilli
; Assembler		:	Yasm 0.8.0
; Description	:	Program to enable and disable speakers
;					on Tandy 1000 SX.					

CPU 8086						; Generate code supported by any PC
BITS 16							; Set 16 bit code generation
ORG 100h						; Code start offset 100h (.COM file)

; Int 21h (DOS) functions
FN_DOS_WR_CHAR_STDOUT	EQU	02h	; Write Character to Standard Output
FN_DOS_WR_STR_STDOUT	EQU	09h	; Write String to Standard Output

; Special characters for strings
TAB						EQU	9	; Horizontal tab
LF						EQU	10	; Line feed       \ Combine to...
CR						EQU	13	; Carriage return / ...get a newline
STOP					EQU	'$'	; DOS string end character

; Command line parameter ID numbers
ID_CMDL_NONE			EQU	0	; No command line parameters, display help
ID_CMDL_BOTH			EQU	1	; Enable both speakers
ID_CMDL_INT				EQU	2	; Enable internal speaker only
ID_CMDL_EXT				EQU	3	; Enable external speaker only

; Port 61h bit flags
FLG_SNDCTRL0			EQU	(1<<5)
FLG_SNDCTRL1			EQU	(1<<6)
FLG_SNDCTRL2			EQU	(1<<4)	; Disables internal speaker when set
MASK_SNDCTRL			EQU	(FLG_SNDCTRL0 | FLG_SNDCTRL1 | FLG_SNDCTRL2)
		

; Section containing code
SECTION .text

;--------------------------------------------------------------------
; Program start
;--------------------------------------------------------------------	
START:
	; Parse command line parameters
	call	GetCmdLine				; Parse command line params
	test	bx, bx					; No parameters?
	jz		.PrintHelp				;  If not, display help

	; Set register bits
	in		al, 61h					; Read port 61h
	and		al, ~MASK_SNDCTRL		; Clear Sound Control bits
	or		al, [bx+g_rgbSpkMask]	; Set new bits
	out		61h, al					; Write port 61h

.ExitToDos:
	; Write current status
	mov		ah, FN_DOS_WR_STR_STDOUT
	mov		dx, g_strSetTo			; Load offset to "set to" string
	int		21h						; Write string to STDOUT
	call	GetStatus				; Get status string to DX
	mov		ah, FN_DOS_WR_STR_STDOUT
	int		21h						; Write string to STDOUT
	cmp		dx, g_strUnknown		; Unknown Sound Control flag mask?
	je		.PrintMask				;  If so, jump to print it

	; Exit to DOS
	mov 	ax, 4C00h				; This dos function exits the program
	int 	21h

; Display help string and exit to DOS
.PrintHelp:
	mov		ah, FN_DOS_WR_STR_STDOUT
	mov		dx, g_strHelp			; Load offset to help string
	int		21h						; Write string to STDOUT
	jmp		.ExitToDos

; Writes Sound Control bit mask to STDOUT
.PrintMask:
	mov		ah, FN_DOS_WR_CHAR_STDOUT
	mov		bx, si					; Copy bit mask to BX
	times 4 shr	bl, 1				; Shift high nybble to BL
	mov		dl, [g_rgbHex+bx]		; Load char to DL
	int		21h						; Print high nybble digit
	and		si, 0Fh					; Clear high nybble
	mov		dl, [g_rgbHex+si]		; Load char to DL
	int		21h						; Print high nybble digit
	mov		dl, 'h'					; Print "h."
	int		21h
	mov		dl, '.'
	int		21h
	mov 	ax, 4C00h				; Exit to DOS
	int 	21h


;--------------------------------------------------------------------
; Reads current speaker status and returns matching string.
; 
; GetStatus
;	Parameters:
;		None
;	Returns:
;		SI:		Sound Control bits from port 61h
;		DX:		Offset to status string
;	Corrupts registers:
;		AX
;--------------------------------------------------------------------
GetStatus:
	in		al, 61h							; Read port 61h
	and		ax, MASK_SNDCTRL				; Clear unwanted bits
	mov		si, ax							; Copy bits to SI
	cmp		al, [g_rgbSpkMask+ID_CMDL_BOTH]	; Set to both?
	je		.RetBoth						;  If so, return
	cmp		al, [g_rgbSpkMask+ID_CMDL_INT]	; Set to internal?
	je		.RetInt							;  If so, return
	cmp		al, [g_rgbSpkMask+ID_CMDL_EXT]	; Set to external?
	je		.RetExt							;  If so, return
	mov		dx, g_strUnknown				; Load offset to "unknown" str
	ret
.RetBoth:
	mov		dx, g_strBoth					; Load offset to "both" str
	ret
.RetInt:
	mov		dx, g_strInt					; Load offset to "internal" str
	ret
.RetExt:
	mov		dx, g_strExt					; Load offset to "external" str
	ret


;--------------------------------------------------------------------
; Parses command line and returns and ID number for parameter.
; 
; Detect_CalculateSize
;	Parameters:
;		None
;	Returns:
;		BX:		Command line parameter ID (see ID_CMDL_ equates)
;	Corrupts registers:
;		AX, SI
;--------------------------------------------------------------------
GetCmdLine:
	xor		ax, ax				; Zero AX
	xor		bx, bx				; Zero BX (ID_CMDL_NONE)
	mov		ah, [80h]			; Load command line char count
	test	ax, ax				; Any command line parameters?
	jz		.Return				;  If not, return with correct status
	mov		si, 81h				; Load offset to first cmd line char
	cld							; LODSB to increment SI
.CmdLineLoop:
	lodsb						; Load cmd line char to AL
	cmp		al, 'e'				; External speaker only?
	je		.RetExternal
	cmp		al, 'i'				; Internal speaker only?
	je		.RetInternal
	cmp		al, 'b'				; Both speakers?
	je		.RetBoth
	cmp		al, ' '				; Skip all whitespace characters
	je		.WhiteSpace
	cmp		al, TAB
	je		.WhiteSpace
	jmp		.Return				; Unknown char, ignore cmd line params
.WhiteSpace:
	dec		ah					; Decrement number of chars left
	jnz		.CmdLineLoop		;  Loop while characters left
	jmp		.Return				; Only whitespace so no params
.RetExternal:
	inc		bx
.RetInternal:
	inc		bx
.RetBoth:
	inc		bx
.Return:
	ret


; Section containing initialized data
SECTION .data

; Lookup table for translating command ID to bit mask for port 61h
g_rgbSpkMask:
	db	0											; ID_CMDL_NONE (never accessed)
	db	FLG_SNDCTRL0 | FLG_SNDCTRL1					; ID_CMDL_BOTH
	db	0											; ID_CMDL_INT (default)
	db	FLG_SNDCTRL0 | FLG_SNDCTRL1 | FLG_SNDCTRL2	; ID_CMDL_EXT

; Hexadecimal printing
g_rgbHex:		db	"0123456789ABCDEF"

; String used in this program
g_strHelp:		db	"Tandy 1000 SX Speaker Multiplexer v1.0.0",LF,CR
				db	"Build 4. October 2009, (c) Tomi Tilli",LF,LF,CR
				
				db	"Configures Tandy 1000 SX speakers.",LF,CR
				db	"TDYSPKR [command]",LF,LF,CR
				
				db	"command",TAB,TAB,"Configures speakers.",LF,CR
				db	TAB,      	TAB,"Note that some games will force their own settings.",LF,CR
				db	TAB,		TAB,"Also note that external speaker cannot be completely",LF,CR
				db	TAB,		TAB,"disabled: 1-voice PC-Speaker sounds will always be heared.",LF,LF,CR

				db	"Following commands are supported:",LF,CR
				db	"internal",	TAB,"Enables internal speaker only (default).",LF,CR
				db	"external",	TAB,"Enables external speaker only.",LF,CR
				db	"both",	TAB,TAB,"Enables internal and external speakers.",LF,LF,CR,STOP

g_strSetTo:		db	"Speakers are now configured to ",STOP
g_strBoth:		db	"both.",STOP
g_strInt:		db	"internal.",STOP
g_strExt:		db	"external.",STOP
g_strUnknown:	db	"unknown: ",STOP
